@@ -33,8 +33,6 @@ int main() {
     float dphi   = 0.003f;  // frequency of sinusoidal frequency drift
 
     float complex x[nfft];
-    char ascii[nfft+1];
-    ascii[nfft] = '\0'; // append null character to end of string
     float nstd = powf(10.0f,noise_floor/20.0f);  // noise standard deviation
     for (n=0; n<num_frames; n++) {
         // generate a frame of data samples
