@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2007 - 2015 Joseph Gaeddert
+ * Copyright (c) 2007 - 2017 Joseph Gaeddert
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -73,8 +73,9 @@ struct ofdmframesync_s {
 
     // gain
     float g0;               // nominal gain (coarse initial estimate)
-    float complex * G0;     // complex subcarrier gain estimate, S0[0]
-    float complex * G1;     // complex subcarrier gain estimate, S0[1]
+    float complex * G0a;    // complex subcarrier gain estimate, S0[a]
+    float complex * G0b;    // complex subcarrier gain estimate, S0[b]
+    float complex * G1;     // complex subcarrier gain estimate, S1
     float complex * G;      // complex subcarrier gain estimate
     float complex * B;      // subcarrier phase rotation due to backoff
     float complex * R;      // 
@@ -206,17 +207,17 @@ ofdmframesync ofdmframesync_create(unsigned int           _M,
 
     // gain
     q->g0 = 1.0f;
-    q->G0 = (float complex*) malloc((q->M)*sizeof(float complex));
-    q->G1 = (float complex*) malloc((q->M)*sizeof(float complex));
-    q->G  = (float complex*) malloc((q->M)*sizeof(float complex));
-    q->B  = (float complex*) malloc((q->M)*sizeof(float complex));
-    q->R  = (float complex*) malloc((q->M)*sizeof(float complex));
+    q->G0a = (float complex*) malloc((q->M)*sizeof(float complex));
+    q->G0b = (float complex*) malloc((q->M)*sizeof(float complex));
+    q->G   = (float complex*) malloc((q->M)*sizeof(float complex));
+    q->B   = (float complex*) malloc((q->M)*sizeof(float complex));
+    q->R   = (float complex*) malloc((q->M)*sizeof(float complex));
 
 #if 1
-    memset(q->G0, 0x00, q->M*sizeof(float complex));
-    memset(q->G1, 0x00, q->M*sizeof(float complex));
-    memset(q->G , 0x00, q->M*sizeof(float complex));
-    memset(q->B,  0x00, q->M*sizeof(float complex));
+    memset(q->G0a, 0x00, q->M*sizeof(float complex));
+    memset(q->G0b, 0x00, q->M*sizeof(float complex));
+    memset(q->G ,  0x00, q->M*sizeof(float complex));
+    memset(q->B,   0x00, q->M*sizeof(float complex));
 #endif
 
     // timing backoff
@@ -299,8 +300,8 @@ void ofdmframesync_destroy(ofdmframesync _q)
     free(_q->s1);
 
     // free gain arrays
-    free(_q->G0);
-    free(_q->G1);
+    free(_q->G0a);
+    free(_q->G0b);
     free(_q->G);
     free(_q->B);
     free(_q->R);
@@ -414,6 +415,11 @@ float ofdmframesync_get_cfo(ofdmframesync _q)
     return nco_crcf_get_frequency(_q->nco_rx);
 }
 
+// set receiver carrier frequency offset estimate
+void ofdmframesync_set_cfo(ofdmframesync _q, float _cfo)
+{
+    nco_crcf_set_frequency(_q->nco_rx, _cfo);
+}
 
 //
 // internal methods
@@ -454,10 +460,10 @@ void ofdmframesync_execute_seekplcp(ofdmframesync _q)
 #endif
 
     // estimate S0 gain
-    ofdmframesync_estimate_gain_S0(_q, &rc[_q->cp_len], _q->G0);
+    ofdmframesync_estimate_gain_S0(_q, &rc[_q->cp_len], _q->G0a);
 
     float complex s_hat;
-    ofdmframesync_S0_metrics(_q, _q->G0, &s_hat);
+    ofdmframesync_S0_metrics(_q, _q->G0a, &s_hat);
     s_hat *= g;
 
     float tau_hat  = cargf(s_hat) * (float)(_q->M2) / (2*M_PI);
@@ -514,10 +520,10 @@ void ofdmframesync_execute_S0a(ofdmframesync _q)
     // TODO : re-estimate nominal gain
 
     // estimate S0 gain
-    ofdmframesync_estimate_gain_S0(_q, &rc[_q->cp_len], _q->G0);
+    ofdmframesync_estimate_gain_S0(_q, &rc[_q->cp_len], _q->G0a);
 
     float complex s_hat;
-    ofdmframesync_S0_metrics(_q, _q->G0, &s_hat);
+    ofdmframesync_S0_metrics(_q, _q->G0a, &s_hat);
     s_hat *= _q->g0;
 
     _q->s_hat_0 = s_hat;
@@ -560,10 +566,10 @@ void ofdmframesync_execute_S0b(ofdmframesync _q)
     windowcf_read(_q->input_buffer, &rc);
 
     // estimate S0 gain
-    ofdmframesync_estimate_gain_S0(_q, &rc[_q->cp_len], _q->G1);
+    ofdmframesync_estimate_gain_S0(_q, &rc[_q->cp_len], _q->G0b);
 
     float complex s_hat;
-    ofdmframesync_S0_metrics(_q, _q->G1, &s_hat);
+    ofdmframesync_S0_metrics(_q, _q->G0b, &s_hat);
     s_hat *= _q->g0;
 
     _q->s_hat_1 = s_hat;
@@ -599,7 +605,7 @@ void ofdmframesync_execute_S0b(ofdmframesync _q)
     float complex g_hat = 0.0f;
     unsigned int i;
     for (i=0; i<_q->M; i++)
-        g_hat += _q->G1[i] * conjf(_q->G0[i]);
+        g_hat += _q->G0b[i] * conjf(_q->G0a[i]);
 
 #if 0
     // compute carrier frequency offset estimate using freq. domain method
@@ -867,7 +873,7 @@ void ofdmframesync_estimate_eqgain(ofdmframesync _q,
         _q->x[i] = (i < _ntaps) ? 1.0f : 0.0f;
     FFT_EXECUTE(_q->fft);
 
-    memmove(_q->G0, _q->G, _q->M*sizeof(float complex));
+    memmove(_q->G0a, _q->G, _q->M*sizeof(float complex));
 
     // smooth complex equalizer gains
     for (i=0; i<_q->M; i++) {
@@ -889,8 +895,8 @@ void ofdmframesync_estimate_eqgain(ofdmframesync _q,
             w = _q->X[(i + _q->M - j) % _q->M];
 
             // accumulate gain
-            //G_hat += w * 0.5f * (_q->G0[j] + _q->G1[j]);
-            G_hat += w * _q->G0[j];
+            //G_hat += w * 0.5f * (_q->G0a[j] + _q->G0b[j]);
+            G_hat += w * _q->G0a[j];
             w0 += w;
         }
 
@@ -1226,8 +1232,8 @@ void ofdmframesync_debug_print(ofdmframesync _q,
     fprintf(fid,"G_hat  = zeros(1,M);\n");
     fprintf(fid,"G      = zeros(1,M);\n");
     for (i=0; i<_q->M; i++) {
-        fprintf(fid,"G0(%3u)    = %12.8f + j*%12.8f;\n", i+1, crealf(_q->G0[i]),   cimagf(_q->G0[i]));
-        fprintf(fid,"G1(%3u)    = %12.8f + j*%12.8f;\n", i+1, crealf(_q->G1[i]),   cimagf(_q->G1[i]));
+        fprintf(fid,"G0(%3u)    = %12.8f + j*%12.8f;\n", i+1, crealf(_q->G0a[i]),  cimagf(_q->G0a[i]));
+        fprintf(fid,"G1(%3u)    = %12.8f + j*%12.8f;\n", i+1, crealf(_q->G0b[i]),  cimagf(_q->G0b[i]));
         fprintf(fid,"G_hat(%3u) = %12.8f + j*%12.8f;\n", i+1, crealf(_q->G_hat[i]),cimagf(_q->G_hat[i]));
         fprintf(fid,"G(%3u)     = %12.8f + j*%12.8f;\n", i+1, crealf(_q->G[i]),    cimagf(_q->G[i]));
     }
